/** 
 * ************************************************************************
 *                                DelaySim                                                               *
 * ************************************************************************
 *  Software package to simulate systems with delays described in the Ph.D. 
 *  thesis
 *  
 *  	"Formal Modeling and Simulation of Biological Systems With Delays" 
 *  
 *  by G.Caravagna, Ph.D. student in Computer Science at the University of 
 *  Pisa, Italy.
 *  
 *  Copyright (C) 2011,  Giulio Caravagna [caravagn@di.unipi.it]
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
**/
package DelaySim;

import java.util.Arrays;
import java.util.Vector;

/**
 * A class describing a reaction-based system to be simulated by the
 * Delay Stochastic Simulation Algorithm where reactions follow the
 * delay-as-duration approach.
 * 
 * @author Giulio Caravagna 
 */
public class DDA 
{
	/**
	 *  the set of reactions both delayed and non-delayed
	 */
	public Reaction[] R;
	
	/**
	 *  the simulation state	
	 */
	public int [] x;
	
	/**
	 *  a private copy of the simulation state
	 */
	private int [] x_orig;
	
	/**
	 * Constructor for a DDA object.
	 * 
	 * @param reactions_d the reactions;
	 * @param state the initial state vector;
	 */
	public DDA(Reaction [] reactions_d, int [] state)
	{
		R = new Reaction[reactions_d.length];
		System.arraycopy(reactions_d, 0, R, 0, R.length);

		x = new int[state.length];
		System.arraycopy(state, 0, x, 0, x.length);
	}

	/**
	 * Execute a DDA simulation for this system starting at time t0 up to time T.
	 * 
	 * @param t0 initial time;
	 * @param T stop time;
	 */
	public void simulate(double t0, double T)
	{
		// local variables used during simulation loop
		double t, a0, tau, minS;
		int j;
		
		// copy initial state, so that you can restore it in the end
		x_orig = new int[x.length];
		System.arraycopy(x, 0, x_orig, 0, x_orig.length); 
		
		// the scheduling list
		Vector<Entry> sched = new Vector<Entry>();	
		
		// initialize a random number generator
		java.util.Random rgen = new java.util.Random(System.currentTimeMillis());
		
		t = t0;
		while(t < T )
		{
			// evaluate all the propensity functions
			a0 = 0;
			for(int i = 0; i < R.length; i++)
				a0 += R[i].rate(x);
			
			// get a value for tau
			if(a0 != 0)
				tau = (Math.log(1.0 / rgen.nextDouble())/ a0);
			else
			{
				// if nothing is scheduled, simulation is forced to stop
				if(sched.isEmpty()) return;
			
				// otherwise, get the biggest possible value for tau (theoretically should be infinity)
				tau = Double.MAX_VALUE;
			}
			
			// get, if any, the next scheduled reaction
			minS = 0;
			if(!sched.isEmpty()) minS = (sched.elementAt(0)).time;
			
			// if next reaction starts before scheduled reactions complete
			if(sched.isEmpty() || t + tau < minS)
			{
				t = t + tau;
				
				// get a value for j
				j = 0;
				double sum_par = a0 - R[0].rate;
				double weight =  rgen.nextDouble() * a0;
				
				for(int i = 1; i < R.length; i++)
				{
				    	if( sum_par < weight )
						break;
					
					sum_par -= R[i].rate;
					j++;
				}
				
				// handle the firing accordingly to the type of reaction
				if( R[j].delay == 0 )
				{
					// use the classical SSA semantics
					R[j].remove_reactants(x);
					R[j].insert_producs(x);
				}
				else
				{
					// use the DDA semantics: remove the reactants
					R[j].remove_reactants(x);
				
					// schedule reaction at time t plus its delay				
					sched.add(new Entry(t + R[j].delay, j));

					// sort the scheduling list
					java.util.Collections.sort(sched);
				}
			}
			else // apply scheduled reaction Rk
			{
				t = minS;
				int k = (sched.elementAt(0)).idx;
				
				// use the DDA semantics: insert the products
				R[k].insert_producs(x);
					
				// remove scheduled element
				sched.removeElementAt(0);				
			}

			System.out.print(t+"\t ");
			for(int i = 0; i < x.length; i++)
				System.out.print(x[i]+"\t ");
			System.out.println("");
		}  

		// restore the copy of initial state
		System.arraycopy(x_orig, 0, x, 0, x.length); 

	}
	
	/**
	 * Print a summary on this system.
	 */
	public void print()
	{
		System.out.print(
				"----------------------------------\n"+
				"        TABLE OF REACTIONS\n" +
				"----------------------------------\n"			
		);
		
		for(int i = 0; i < R.length; i++)
			System.out.println(R[i]);

		System.out.print(
				"----------------------------\n"+
				"        SYSTEM STATE\n" +
				"----------------------------\n"			
		);
		
		System.out.print("\t"+Arrays.toString(x));
		
		System.out.println("\n----------------------------");
		
	}
}
