/** 
 * ************************************************************************
 *                                DelaySim                                                               *
 * ************************************************************************
 *  Software package to simulate systems with delays described in the Ph.D. 
 *  thesis
 *  
 *  	"Formal Modeling and Simulation of Biological Systems With Delays" 
 *  
 *  by G.Caravagna, Ph.D. student in Computer Science at the University of 
 *  Pisa, Italy.
 *  
 *  Copyright (C) 2011,  Giulio Caravagna [caravagn@di.unipi.it]
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
**/

package DelaySim;

import java.util.Arrays;

/**
 * A class describing a generic reaction. This reaction is described by
 * - a name;
 * - a stoichiometry vector for reactants, and one for products;
 * - a kinetic constant;
 * - a delay, which can be either greater or equal to 0.
 * 
 * @author Giulio Caravagna 
 */
public class Reaction 
{
	/**
	* the stoichiometry vector for reactants
	*/
	public int [] nu_r;
	
	/**
	* the stoichiometry vector for products
	*/
	public int [] nu_p;
	
	/**
	* the kinetic constant
	*/
	public double k;
	
	/**
	* the delay
	*/
	public double delay;
		
	/**
	* the reaction name
	*/
	public String name;
	
	private int idx_molecule_1;
	private int idx_molecule_2;
	public double rate;
	
	/**
	 * Constructor for a Reaction object.
	 * 
	 * @param name the name of the reaction;
	 * @param reactants the stoichiometry vector for reactants;
	 * @param products  the stoichiometry vector for products;
	 * @param constant the kinetic constant;
	 * @param delay the delay of the reaction.
	 */
	public Reaction(String name, int [] reactants, int [] products, double constant, double delay)
	{
		if(
				reactants.length != products.length ||
				constant <= 0 ||
				delay < 0
				)
		{
			System.out.println("Bad-formed reaction "+name+". Aborting.");
			System.exit(-1);
		}
			
		this.name = name;
		
		// add reactants and products
		nu_r = new int[reactants.length];
		System.arraycopy(reactants, 0, nu_r, 0, reactants.length);
		
		// initialize the mapping
		idx_molecule_1 = idx_molecule_2 = -1;
		init_mapping();
		
		nu_p = new int[products.length];
		System.arraycopy(products, 0, nu_p, 0, products.length);
		
		// set kinetics
		this.k = constant;
		this.delay = delay;
	}
	
	/**
	 * Create a mapping to speed-up the evaluation of the
	 * propensity function
	 */
	private void init_mapping()
	{
		for(int i = 0; i < nu_r.length; i++)
		{
			// it consumes i-th type of molecule
			if(nu_r[i] != 0)
			{
				if(idx_molecule_1 == -1)
					idx_molecule_1 = i;
				else
					idx_molecule_2 = i;					
			}			
		}
	}
	
	/**
	 * Evaluate the propensity function for the reaction in the state described
	 * by the vector used as parameter. The analytical form of the propensity function
	 * is the classical by Gillespie.
	 * 
	 * @param state the integer vector describing the state;
	 * 
	 * @return the evaluation of the propensity function.
	 */
	public double rate( int [] state )
	{
		if(idx_molecule_1 == -1 && idx_molecule_2 == -1) rate = k; 
		else
			if(idx_molecule_2 == -1) rate = k * state[idx_molecule_1]; 
			else				
				if(nu_r[idx_molecule_1] == -2)
					rate = k * state[idx_molecule_1] * (state[idx_molecule_1] - 1);
				else
					rate = k * state[idx_molecule_1] * state[idx_molecule_2];
	
		return rate;
	}
	
	/**
	 * Check if reaction is applicable in the state vector given as parameter.
	 * 
	 * @param state the integer vector describing the state;
	 */
	public boolean is_applicable( int [] state )
	{
		for(int i = 0; i < nu_r.length; i++)
		{
			// this is actually a subtraction since nu_r[i] < 0
			if( state[i] + nu_r[i] < 0)  
				return false;
		}
		
		return true;
	}
	
	/**
	 * Perform removal of the reactants from the state vector given as parameter.
	 * 
	 * @param state the integer vector describing the state;
	 */
	public void remove_reactants( int [] state )
	{
		for(int i = 0; i < state.length; i++)
			state[i] += nu_r[i];
	}
	
	/**
	 * Perform insertion of the products in the state vector given as parameter.
	 * 
	 * @param state the integer vector describing the state;
	 */
	public void insert_producs( int [] state )
	{
		for(int i = 0; i < state.length; i++)
			state[i] += nu_p[i];
	}

	/**
	 * @return a string representation of this reaction
	 */
	public String toString()
	{
		return name +": "+Arrays.toString(nu_r) + " ===== "+k+", "+delay+" ====> "+Arrays.toString(nu_p);
	}
	

}
