/** 
 * ************************************************************************
 *                                DelaySim                                                               *
 * ************************************************************************
 *  Software package to simulate systems with delays described in the Ph.D. 
 *  thesis
 *  
 *  	"Formal Modeling and Simulation of Biological Systems With Delays" 
 *  
 *  by G.Caravagna, Ph.D. student in Computer Science at the University of 
 *  Pisa, Italy.
 *  
 *  Copyright (C) 2011,  Giulio Caravagna [caravagn@di.unipi.it]
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
**/
package DelaySim.examples;

import DelaySim.DDA;
import DelaySim.PDA;
import DelaySim.Reaction;

/**
 * The cell-cycle model with a delay in the passage from the interphase to mitothic phase. 
 * 
 * @author Giulio Caravagna 
 */
public class CellCycleModel {

	/*
	 * Change this to change the initial state vector
	 */
	public static int INITAL_TI = 10;
	public static int INITAL_TM = 10;
	
	public static void main(String[] args) 
	{		
		/* Full model parameters (Table 3.2)
		 * 
		 * PARAM   R-I      R-II     R-III    R-IV
		 * a1      0.6      0.4      1.0      0.8 
		 * d       0.6      1.0      1.8      0.8 
		 * a4      0.5      0.5      0.5      0.5 
		 * d2      0.3      0.3      0.3      0.3 
		 * 
		 * where d = d3 + a4 
		 * 
		 * and delays
		 * 
		 * sigma = {0.0, 1.0, 10.0}
		 */
		
		// the model parameters for region R-IV and delay=10.0
		double a1 = 0.8;
		double d = 0.8;
		double a4 = 0.5;
		double d2 = 0.3;
		double sigma = 10.0;
		
		// simulation time 
		double time = 5;
		
		/* Reaction R1: T_I -> T_M 
		 * with kinetic constant a1 and delay sigma
		 */
		int [] v1_R = {-1, 0}; 
		int [] v1_P = {0, 1};   
		Reaction r1 = new Reaction("R1", v1_R, v1_P, a1, sigma);

		/* Reaction R2: T_I -> T_M 
		 * with kinetic constant a1 and delay 0
		 */		
		int [] v2_R = {0, -1}; 
		int [] v2_P = {2, 0};   
		Reaction r2 = new Reaction("R2", v2_R, v2_P, a4, 0.0);

		/* Reaction R3: T_I ->  
		 * with kinetic constant d2 and delay 0
		 */		
		int [] v3_R = {-1, 0}; 
		int [] v3_P = {0, 0};   
		Reaction r3 = new Reaction("R3", v3_R, v3_P, d2, 0.0);

		
		/* Reaction R4: T_M ->  
		 * with kinetic constant (d-a4) and delay 0
		 */		
		int [] v4_R = {0, -1}; 
		int [] v4_P = {0, 0};   
		Reaction r4 = new Reaction("R4", v4_R, v4_P, d-a4, 0.0);

		
		/*
		 *  The initial state is X(t0) = ( INITIAL_TI, INITIAL_TM )
		 */				
		int [] x0 = {INITAL_TI, INITAL_TM};
		
		/*
		 *  The set of all the reactions in this system
		 */
		Reaction [] reactions = {r1, r2, r3, r4};
		
		/*
		 * The DDA system 
		 */
		DDA DDA_system = new DDA(reactions, x0);		
		DDA_system.print();
		
		/*
		 * A DDA simulation starting at time t0 = 0 and going for 'time' time units
		 */
		DDA_system.simulate(0, time);
		

		/*
		 * The PDA system 
		 */
		PDA PDA_system = new PDA(reactions, x0);		
		PDA_system.print();
		
		/*
		 * A PDA simulation starting at time t0 = 0 and going for 'time' time units
		 */
		PDA_system.simulate(0, time);
	}
}
